#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import subprocess
from collections import namedtuple

# git_access, , access to components available from git repositories, internal
from yotta.lib import git_access
# fsutils, , misc filesystem utils, internal
from yotta.lib import fsutils
# version, , represent versions and specifications, internal
from yotta.lib import version
# sourceparse, , parse version source urls, internal
from yotta.lib import sourceparse
# install, , install components, internal
from yotta import install


Test_Name = 'testing-dummy'
Test_Repo = "git@github.com:autopulated/testing-dummy.git"
Test_Repo_With_Spec = "git@github.com:autopulated/testing-dummy.git#0.0.1"
Test_Deps_Name = 'git-access-testing'
Test_Deps_Target = 'x86-osx-native,*'


def ensureGitConfig():
    # test if we have a git user set up, if not we need to set one
    child = subprocess.Popen([
            'git','config', '--global', 'user.email'
        ], stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    out, err = child.communicate()
    if not len(out):
        commands = [
            ['git','config', '--global', 'user.email', 'test@yottabuild.org'],
            ['git','config', '--global', 'user.name', 'Yotta Test']
        ]
        for cmd in commands:
            child = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = child.communicate()


class TestGitAccess(unittest.TestCase):
    def setUp(self):
        ensureGitConfig()
        vs = sourceparse.parseSourceURL(Test_Repo)
        self.remote_component = git_access.GitComponent.createFromSource(vs, Test_Name)
        self.assertTrue(self.remote_component)
        self.working_copy = self.remote_component.clone()
        self.assertTrue(self.working_copy)

    def tearDown(self):
        fsutils.rmRf(self.working_copy.directory)

    def test_availableVersions(self):
        versions = self.working_copy.availableVersions()
        self.assertIn(version.Version('v0.0.1'), versions)

    def test_versionSpec(self):
        vs = sourceparse.parseSourceURL(Test_Repo_With_Spec)
        spec = git_access.GitComponent.createFromSource(vs, Test_Name).versionSpec()
        v = spec.select(self.working_copy.availableVersions())
        self.assertTrue(v)

    def test_installDeps(self):
        Args = namedtuple('Args', ['component', 'target', 'act_globally', 'install_linked', 'install_test_deps'])
        install.installComponent(Args(Test_Deps_Name, Test_Deps_Target, False, False, 'own'))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import tempfile
import os

# validate, , validate various things, internal
from yotta.lib import settings
from yotta.lib.fsutils import rmRf

class TestSettings(unittest.TestCase):
    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        test_files = [
            ('1.json', '{"a":{"b":{"c":"1-value"}}}'),
            ('2.json', '{"a":{"b":{"c":"2-value"}, "b2":"2-value"}}'),
            ('3.json', '{"a":{"b":{"c":"3-value"}, "b2":"3-value"}, "a2":"3-value"}')
        ]
        self.filenames = []
        for fn, s in test_files:
            self.filenames.append(os.path.join(self.test_dir, fn))
            with open(self.filenames[-1], 'w') as f:
                f.write(s)

    def tearDown(self):
        rmRf(self.test_dir)

    def test_merging(self):
        p = settings._JSONConfigParser()
        p.read(self.filenames)
        self.assertEqual(p.get('a.b.c'), '1-value')
        self.assertEqual(p.get('a.b2'), '2-value')
        self.assertEqual(p.get('a2'), '3-value')

    def test_setting(self):
        p = settings._JSONConfigParser()
        p.read(self.filenames)

        p.set('foo', 'xxx')
        self.assertEqual(p.get('foo'), 'xxx')

        p.set('someLongNameHere_etc_etc', 'xxx')
        self.assertEqual(p.get('someLongNameHere_etc_etc'), 'xxx')

        p.set('someLongNameHere_etc_etc.with.a.path', True, filename=self.filenames[1])
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.a.path'), True)

        p.set('someLongNameHere_etc_etc.with.a.path', False, filename=self.filenames[1])
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.a.path'), False)

        # NB: don't expect it to change when we set a value that's shadowed by
        # an earlier file:
        p.set('someLongNameHere_etc_etc.with.a.path', 7, filename=self.filenames[2])
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.a.path'), False)

        p.set('someLongNameHere_etc_etc.with.another.path', 7, filename=self.filenames[2])
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.another.path'), 7)


    def test_writing(self):
        p = settings._JSONConfigParser()
        p.read(self.filenames)

        p.set('foo', 'xxx')
        p.set('someLongNameHere_etc_etc', 'xxx')
        p.set('someLongNameHere_etc_etc.with.a.path', True, filename=self.filenames[1])
        p.set('someLongNameHere_etc_etc.with.a.path', False, filename=self.filenames[1])
        p.set('someLongNameHere_etc_etc.with.a.path', 7, filename=self.filenames[2])
        p.set('someLongNameHere_etc_etc.with.another.path', 7, filename=self.filenames[2])

        # NB: only write settings to the first file
        p.write()

        self.assertEqual(p.get('foo'), 'xxx')
        self.assertEqual(p.get('someLongNameHere_etc_etc'), 'xxx')
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.a.path'), False)
        self.assertEqual(p.get('someLongNameHere_etc_etc.with.another.path'), 7)

        p2 = settings._JSONConfigParser()
        p2.read(self.filenames)
        self.assertEqual(p2.get('foo'), 'xxx')
        self.assertEqual(p2.get('someLongNameHere_etc_etc'), 'xxx')

        # check that we only wrote settings to the first file
        self.assertEqual(p2.get('someLongNameHere_etc_etc.with.a.path'), None)

        # now write settings for the other files, and continue
        p.write(self.filenames[1])
        p.write(self.filenames[2])

        p3 = settings._JSONConfigParser()
        p3.read(self.filenames)
        self.assertEqual(p3.get('someLongNameHere_etc_etc.with.a.path'), False)
        self.assertEqual(p3.get('someLongNameHere_etc_etc.with.another.path'), 7)


        p4 = settings._JSONConfigParser()
        p4.read([self.filenames[1]])
        self.assertEqual(p4.get('foo'), None)
        self.assertEqual(p4.get('someLongNameHere_etc_etc.with.a.path'), False)
        self.assertEqual(p4.get('someLongNameHere_etc_etc.with.another.path'), None)

        p5 = settings._JSONConfigParser()
        p5.read([self.filenames[2]])
        self.assertEqual(p5.get('foo'), None)
        self.assertEqual(p5.get('someLongNameHere_etc_etc.with.a.path'), 7)
        self.assertEqual(p5.get('someLongNameHere_etc_etc.with.another.path'), 7)



if __name__ == '__main__':
    unittest.main()







# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
from __future__ import print_function
import logging
import os
import json

# colorama, BSD 3-Clause license, cross-platform terminal colours, pip install colorama
import colorama

# validate, , validate things, internal
from .lib import validate
# access, , get components (and check versions), internal
from .lib import access
# fsutils, , misc filesystem utils, internal
from .lib import fsutils

def addOptions(parser):
    parser.add_argument('--all', '-a', dest='show_all', default=False, action='store_true',
        help='Show all dependencies (including repeats, and test-only dependencies)'
    )
    parser.add_argument('--json', '-j', dest='json', default=False, action='store_true',
        help='Output json representation of dependencies.'
    )

def execCommand(args, following_args):
    c = validate.currentDirectoryModule()
    if not c:
        return 1

    if not args.target:
        logging.error('No target has been set, use "yotta target" to set one.')
        return 1

    target, errors = c.satisfyTarget(args.target)
    if errors:
        for error in errors:
            logging.error(error)
        return 1

    dependencies = c.getDependenciesRecursive(
                      target = target,
        available_components = [(c.getName(), c)],
                        test = True
    )
    if args.json:
        putln(formatJsonDeps(target, dependencies, args.show_all))
    else:
        putln(
            ComponentDepsFormatter(
                               target = target,
                 available_components = dependencies,
                                plain = args.plain,
                             list_all = args.show_all
            ).format(
                c, [c.getName()]
            )
        )

def formatJsonDeps(target, available_components, list_all):
    d = {}
    for c in available_components:
        co = available_components[c]
        d[c] = {}
        d[c]['name'] = co.getName()
        d[c]['version'] = str(co.getVersion())
        d[c]['dependencies'] = {}
        for dep in co.getDependencySpecs(target=target):
            depcomp = available_components[dep.name]
            spec = access.remoteComponentFor(dep.name, dep.version_req, 'modules').versionSpec()
            dd = {
                'verspec':dep.version_req,
                   'test':dep.is_test_dependency,
                'missing':not available_components[dep.name],
               'mismatch':not spec.match(depcomp.getVersion()),
                   'link':co.installedLinked()
            }
            d[c]['dependencies'][dep.name]=dd
    return json.dumps(d)

def islast(generator):
    next_x = None
    first = True
    for x in generator:
        if not first:
            yield (next_x, False)
        next_x = x
        first = False
    if not first:
        yield (next_x, True)

def putln(x):
    if u'unicode' in str(type(x)):
        # python 2.7
        print(x.encode('utf-8'))
    else:
        print(x)

def relpathIfSubdir(path):
    relpath = os.path.relpath(path)
    if relpath.startswith('..'):
        return path
    else:
        return relpath

class ComponentDepsFormatter(object):
    def __init__(self, target=None, available_components=None, list_all=False, plain=False):
        # don't even try to do Unicode on windows. Even if we can encode it
        # correctly, the default terminal fonts don't support Unicode
        # characters :(
        self.use_unicode = not ((os.name == 'nt') or plain)
        self.use_colours = not plain
        self.target    = target
        self.list_all  = list_all
        self.available = available_components
        if plain:
            self.L_Char = u' '
            self.T_Char = u' '
            self.Dash_Char = u' '
            self.Pipe_Char = u' '
        elif self.use_unicode:
            self.L_Char = u'\u2517'
            self.T_Char = u'\u2523'
            self.Dash_Char = u'\u2501'
            self.Pipe_Char = u'\u2503'
        else:
            self.L_Char = u'\\'
            self.T_Char = u'|'
            self.Dash_Char = u'_'
            self.Pipe_Char = u'|'
        super(ComponentDepsFormatter, self).__init__()

    def format(
        self,
        component,
        processed,
        indent=u'',
        tee=u'',
        installed_at=u'',
        test_dep=False,
        spec=None
    ):
        r = u''

        if self.use_colours:
            DIM    = colorama.Style.DIM       #pylint: disable=no-member
            BRIGHT = colorama.Style.BRIGHT    #pylint: disable=no-member
            GREEN  = colorama.Fore.GREEN      #pylint: disable=no-member
            RED    = colorama.Fore.RED        #pylint: disable=no-member
            RESET  = colorama.Style.RESET_ALL #pylint: disable=no-member
        else:
            DIM = BRIGHT = GREEN = RED = RESET = u''

        mods_path = component.modulesPath()
        deps = component.getDependencies(
                available_components = self.available,
                              target = self.target,
                                test = True,
                            warnings = False
        )
        specs = dict([(x.name, x) for x in component.getDependencySpecs(target=self.target)])

        def isTestOnly(name):
            return specs[name].is_test_dependency

        def shouldDisplay(x):
            if self.list_all:
                # list everything everywhere (apart from test dependencies of test
                # dependencies, which should be considered irrelevant)
                if component.isTestDependency() and isTestOnly(x[0]):
                    return False
                else:
                    return True
            if (not isTestOnly(x[0]) or not len(indent)):
                # this is non-test dependency, or a top-level test dependency
                if not x[1]:
                    # if it's missing, display it
                    return True
                if x[1].path == os.path.join(mods_path, x[0]):
                    # if it's installed in this module, display it
                    return True
                if x[0] in deps_here:
                    # if it's first depended on by this module, then display it
                    return True
            # everything else shouldn't be displayed here
            return False


        line = indent[:-2] + tee + component.getName() + u' ' + DIM + str(component.getVersion()) + RESET

        if spec and not spec.match(component.getVersion()):
            line += u' ' + RESET + BRIGHT + RED + str(spec) + RESET
        if test_dep:
            line += u' ' + DIM + u'(test dependency)' + RESET
        if len(installed_at):
            line += u' ' + DIM + installed_at + RESET
        if component.installedLinked():
            line += GREEN + BRIGHT + u' -> ' + RESET + GREEN + fsutils.realpath(component.path) + RESET

        r += line + '\n'

        deps_here  = [x for x in list(deps.keys()) if (x not in processed)]
        print_deps = [x for x in list(deps.items()) if shouldDisplay(x)]

        processed += [x[0] for x in print_deps]


        for (name, dep), last in islast(print_deps):
            if last:
                next_indent = indent + u'  '
                tee = self.L_Char + self.Dash_Char + u' '
                next_tee = self.L_Char + self.Dash_Char + u' '
            else:
                next_indent = indent + self.Pipe_Char + u' '
                tee = self.T_Char + self.Dash_Char + u' '
                next_tee = self.T_Char + self.Dash_Char + u' '
            test_dep_status = u''
            if isTestOnly(name):
                test_dep_status = u' (test dependency)'

            if not dep:
                r += indent + tee + name + u' ' + specs[name].version_req + test_dep_status + BRIGHT + RED + ' missing' + RESET + '\n'
            else:
                spec = access.remoteComponentFor(name, specs[name].version_req, 'modules').versionSpec()
                if not spec:
                    spec_descr = u''
                elif spec.match(dep.getVersion()):
                    spec_descr = u' ' + str(spec)
                else:
                    spec_descr = u' ' + RESET + BRIGHT + RED + str(spec)
                spec_descr += test_dep_status

                if name in deps_here:
                    # dependencies that are first used here may actually be
                    # installed higher up our dependency tree, if they are,
                    # illustrate that:
                    if dep.path == os.path.join(mods_path, name):
                        r += self.format(
                                       dep,
                                 processed,
                               next_indent,
                                  next_tee,
                                  test_dep = isTestOnly(name),
                                      spec = spec
                        )
                    else:
                        r += self.format(
                                       dep,
                                 processed,
                               next_indent,
                                  next_tee,
                              installed_at = relpathIfSubdir(dep.path),
                                  test_dep = isTestOnly(name),
                                      spec = spec
                        )
                else:
                    r += indent + tee + DIM + name + spec_descr + RESET + '\n'
        return r



# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# re-implementation of the pycrypto exportKey method for openSSH, to work
# around python 3 bug

# standard library modules
import struct
import binascii
import sys

if sys.version_info[0] == 3:
    def bord(x):
        return x
    def bchr(x):
        return bytes([x])

elif sys.version_info[0] == 2:
    def bord(x):
        return ord(x)
    def bchr(x):
        return chr(x)

# long_to_bytes: from PyCrypto,public domain, a vailable at:
# https://github.com/dlitz/pycrypto/blob/7acba5f3a6ff10f1424c309d0d34d2b713233019/lib/Crypto/Util/number.py
def long_to_bytes(n, blocksize=0):
    """long_to_bytes(n:long, blocksize:int) : string
    Convert a long integer to a byte string.

    If optional blocksize is given and greater than zero, pad the front of the
    byte string with binary zeros so that the length is a multiple of
    blocksize.
    """
    # after much testing, this algorithm was deemed to be the fastest
    s = b''
    n = int(n)
    pack = struct.pack
    while n > 0:
        s = pack('>I', n & 0xffffffff) + s
        n = n >> 32
    # strip off leading zeros
    for i in range(len(s)):
        if s[i] != '\000':
            break
    else:
        # only happens when n == 0
        s = '\000'
        i = 0
    s = s[i:]
    # add back some pad bytes.  this could be done more efficiently w.r.t. the
    # de-padding being done above, but sigh...
    if blocksize > 0 and len(s) % blocksize:
        s = (blocksize - len(s) % blocksize) * '\000' + s
    return s

def openSSH(pubkey):
    e = long_to_bytes(pubkey.e)
    n = long_to_bytes(pubkey.n)

    if bord(e[0]) & 0x80:
        e = bchr(0) + e

    if bord(n[0]) & 0x80:
        n = bchr(0) + n

    key_bytes = b'ssh-rsa' + struct.pack('>I', len(e)) + e + struct.pack('>I', len(n)) + n

    return b'ssh-rsa ' + binascii.b2a_base64(key_bytes)[:-1]



# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# yotta's error handling utilities

# standard library modules, , ,
import sys

def _addExceptHook(fn):
    # standard library modules, , ,
    import functools
    previous_excepthook = sys.excepthook
    @functools.wraps(fn)
    def chainExceptHook(*args, **kwargs):
        fn(*args, **kwargs)
        return previous_excepthook(*args, **kwargs)
    sys.excepthook = chainExceptHook


def _yottaExceptHook(exc_type, exc_val, exc_tb):
    try:
        import pkg_resources
        sys.stderr.write('Fatal Exception, yotta=' + pkg_resources.require("yotta")[0].version + '\n')
    except Exception as e:
        sys.stderr.write('Fatal Exception, yotta=unknown\n')
_addExceptHook(_yottaExceptHook)



#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
import os
import tempfile

# internal modules:
from yotta.lib.fsutils import mkDirP, rmRf
from yotta.lib.detect import systemDefaultTarget
from yotta.lib import component
from .cli import cli

Test_Files = {
    '.yotta_ignore': '''
#comment
/moo
b/c/d
b/c/*.txt
/a/b/test.txt
b/*.c
/source/a/b/test.txt
/test/foo
sometest/a
someothertest
ignoredbyfname.c
''',
    'module.json': '''
{
  "name": "test-testdep-f",
  "version": "0.0.6",
  "description": "Module to test test-dependencies and ignoring things",
  "author": "autopulated",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "dependencies": {},
  "testDependencies": {}
}
''',

'a/b/c/d/e/f/test.txt': '',
'a/b/c/d/e/test.c': '#error should be ignored',
'a/b/c/d/e/test.txt': '',
'a/b/c/d/test.c': '#error should be ignored',
'a/b/c/d/test.txt': '',
'a/b/c/d/z/test.c':'#error should be ignored',
'a/b/c/test.txt': '',
'a/b/test.txt':'',
'a/test.txt':'',
'comment':'# should not be ignored',
'f/f.h':'''
#ifndef __F_H__
#define __F_H__
int f();
#endif
''',
'source/moo/test.txt':'',
'source/a/b/c/d/e/f/test.txt': '',
'source/a/b/c/d/e/test.c': '#error should be ignored',
'source/a/b/c/d/e/test.txt': '',
'source/a/b/c/d/test.c': '#error should be ignored',
'source/a/b/c/d/test.txt': '',
'source/a/b/c/d/z/test.c':'#error should be ignored',
'source/a/b/c/test.txt': '',
'source/a/b/test.txt':'',
'source/a/test.txt':'',
'source/f.c':'''

int f(){
    return 6;
}
''',
'test/anothertest/ignoredbyfname.c':'#error should be ignored',
'test/anothertest/ignoredbyfname.c':'''
#include <stdio.h>

#include "f/f.h"

int main(){
    int result = f();
    printf("%d\n", result);
    return !(result == 6);
}
''',
'test/foo/ignored.c':'''
#error should be ignored
''',
'test/someothertest/alsoignored.c':'''
#error should be ignored
''',
'test/sometest/a/ignored.c':'''
#error should be ignored
'''
}

Default_Test_Files = {
    'module.json': '''
{
  "name": "test-testdep-f",
  "version": "0.0.6",
  "license": "Apache-2.0"
}'''
}

def isWindows():
    # can't run tests that hit github without an authn token
    return os.name == 'nt'

def writeTestFiles(files):
    test_dir = tempfile.mkdtemp()
    for path, contents in files.items():
        path_dir, file_name =  os.path.split(path)
        path_dir = os.path.join(test_dir, path_dir)
        mkDirP(path_dir)
        with open(os.path.join(path_dir, file_name), 'w') as f:
            f.write(contents)
    return test_dir

class TestPackIgnores(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_dir = writeTestFiles(Test_Files)

    @classmethod
    def tearDownClass(cls):
        rmRf(cls.test_dir)

    def test_absolute_ignores(self):
        c = component.Component(self.test_dir)
        self.assertTrue(c.ignores('moo'))
        self.assertTrue(c.ignores('test/foo/ignored.c'))

    def test_glob_ignores(self):
        c = component.Component(self.test_dir)
        self.assertTrue(c.ignores('a/b/c/test.txt'))
        self.assertTrue(c.ignores('a/b/test.txt'))
        self.assertTrue(c.ignores('a/b/test.c'))
        self.assertTrue(c.ignores('source/a/b/c/test.txt'))
        self.assertTrue(c.ignores('source/a/b/test.txt'))
        self.assertTrue(c.ignores('source/a/b/test.c'))

    def test_relative_ignores(self):
        c = component.Component(self.test_dir)
        self.assertTrue(c.ignores('a/b/c/d/e/f/test.txt'))
        self.assertTrue(c.ignores('a/b/test.txt'))
        self.assertTrue(c.ignores('source/a/b/c/d/e/f/test.txt'))
        self.assertTrue(c.ignores('source/a/b/test.txt'))
        self.assertTrue(c.ignores('test/anothertest/ignoredbyfname.c'))
        self.assertTrue(c.ignores('test/someothertest/alsoignored.c'))

    def test_default_ignores(self):
        default_test_dir = writeTestFiles(Default_Test_Files)
        c = component.Component(default_test_dir)
        self.assertTrue(c.ignores('.something.c.swp'))
        self.assertTrue(c.ignores('.something.c~'))
        self.assertTrue(c.ignores('path/to/.something.c.swm'))
        self.assertTrue(c.ignores('path/to/.something.c~'))
        self.assertTrue(c.ignores('.DS_Store'))
        self.assertTrue(c.ignores('.git'))
        self.assertTrue(c.ignores('.hg'))
        self.assertTrue(c.ignores('.svn'))
        self.assertTrue(c.ignores('yotta_modules'))
        self.assertTrue(c.ignores('yotta_targets'))
        self.assertTrue(c.ignores('build'))
        self.assertTrue(c.ignores('.yotta.json'))

        rmRf(default_test_dir)

    def test_comments(self):
        c = component.Component(self.test_dir)
        self.assertFalse(c.ignores('comment'))

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_build(self):
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'clean'], self.test_dir)
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'build'], self.test_dir)
        self.assertNotIn('ignoredbyfname', stdout)
        self.assertNotIn('someothertest', stdout)
        self.assertNotIn('sometest', stdout)

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_test(self):
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'clean'], self.test_dir)
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], self.test_dir)
        self.assertNotIn('ignoredbyfname', stdout)
        self.assertNotIn('someothertest', stdout)
        self.assertNotIn('sometest', stdout)

    def runCheckCommand(self, args, test_dir):
        stdout, stderr, statuscode = cli.run(args, cwd=self.test_dir)
        if statuscode != 0:
            print('command failed with status %s' % statuscode)
            print(stdout)
            print(stderr)
        self.assertEqual(statuscode, 0)
        return stdout or stderr

if __name__ == '__main__':
    unittest.main()






#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest
from collections import namedtuple

# settings, , load and save settings, internal
from yotta.lib import settings
# globalconf, share global arguments between modules, internal
from yotta.lib import globalconf
# install, , install components, internal
from yotta import install

globalconf.set('interactive', False)

Test_Name = 'testing-dummy'
Test_Deps_Name = "autopulated/github-access-testing"
Test_Branch_Name = "autopulated/github-access-testing#master"
Test_Deps_Target = "x86-osx-native,*"

def hasGithubConfig():
    # can't run tests that hit github without an authn token
    if not settings.getProperty('github', 'authtoken'):
        return False
    return True

class TestGitHubAccess(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_installDeps(self):
        Args = namedtuple('Args', ['component', 'target', 'act_globally', 'install_linked', 'install_test_deps'])
        install.installComponent(Args(Test_Deps_Name, Test_Deps_Target, False, False, 'own'))

    @unittest.skipIf(not hasGithubConfig(), "a github authtoken must be specified for this test (run yotta login, or set YOTTA_GITHUB_AUTHTOKEN)")
    def test_branchAccess(self):
        Args = namedtuple('Args', ['component', 'target', 'act_globally', 'install_linked', 'install_test_deps'])
        install.installComponent(Args(Test_Branch_Name, Test_Deps_Target, False, False, 'own'))


if __name__ == '__main__':
    unittest.main()





# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import logging

# validate, , validate things, internal
from .lib import validate


def addOptions(parser):
    parser.add_argument('program', default=None,
        help='name of the program to be debugged'
    )


def execCommand(args, following_args):
    cwd = os.getcwd()

    c = validate.currentDirectoryModule()
    if not c:
        return 1

    target, errors = c.satisfyTarget(args.target)
    if errors:
        for error in errors:
            logging.error(error)
        return 1

    builddir = os.path.join(cwd, 'build', target.getName())

    # !!! FIXME: the program should be specified by the description of the
    # current project (or a default value for the program should)
    errcode = None
    error = target.debug(builddir, args.program)
    if error:
        logging.error(error)
        errcode = 1

    return errcode





# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import json
import os
import stat
from collections import OrderedDict

# provide read & write methods for json files that maintain the order of
# dictionary keys, and indent consistently

# Internals
def load(path):
    with open(path, 'r') as f:
        # using an ordered dictionary for objects so that we preserve the order
        # of keys in objects (including, for example, dependencies)
        return json.load(f, object_pairs_hook=OrderedDict)

def dump(path, obj):
    with os.fdopen(os.open(path, os.O_WRONLY | os.O_CREAT, stat.S_IRUSR | stat.S_IWUSR), 'w') as f:
        os.chmod(path, stat.S_IRUSR | stat.S_IWUSR)
        json.dump(obj, f, indent=2, separators=(',', ': '))
        f.write(u'\n')
        f.truncate()

def loads(string):
    return json.loads(string, object_pairs_hook=OrderedDict)

def dumps(obj):
    return json.dumps(obj, indent=2, separators=(',', ': '))



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import signal
import subprocess
import logging
import string
import traceback
import errno
import itertools
import shlex
from collections import OrderedDict

# Ordered JSON, , read & write json, internal
import ordered_json
# access, , get components, internal
import access
import access_common
# Pack, , common parts of Components/Targets, internal
import pack
# fsutils, , misc filesystem utils, internal
import fsutils

Target_Description_File = 'target.json'
App_Config_File = 'config.json'
Registry_Namespace = 'targets'
Schema_File = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'schema', 'target.json')

logger = logging.getLogger('target')

def _ignoreSignal(signum, frame):
    logger.debug('ignoring signal %s, traceback:\n%s' % (
        signum, ''.join(traceback.format_list(traceback.extract_stack(frame)))
    ))

def _newPGroup():
    os.setpgrp()

def _mergeDictionaries(*args):
    ''' merge dictionaries of dictionaries recursively, with elements from
        dictionaries earlier in the argument sequence taking precedence
    '''
    # to support merging of OrderedDicts, copy the result type from the first
    # argument:
    result = type(args[0])()
    for k, v in itertools.chain(*[x.items() for x in args]):
        if not k in result:
            result[k] = v
        elif isinstance(result[k], dict) and isinstance(v, dict):
            result[k] = _mergeDictionaries(result[k], v)
    return result

# API

def getDerivedTarget(
        target_name_and_version,
                   targets_path,
                application_dir = None,
                install_missing = True,
               update_installed = False
    ):
    ''' Get the specified target description, optionally ensuring that it (and
        all dependencies) are installed in targets_path.

        Returns (DerivedTarget, errors), or (None, errors) if the leaf target
        could not be found/installed.
    '''
    logger.debug('satisfy target: %s' % target_name_and_version);
    if ',' in target_name_and_version:
        name, ver = target_name_and_version.split(',')
        dspec = pack.DependencySpec(name, ver)
    else:
        dspec = pack.DependencySpec(target_name_and_version, "*")

    leaf_target      = None
    previous_name    = dspec.name
    search_dirs      = [targets_path]
    target_hierarchy = []
    errors           = []
    while True:
        t = None
        try:
            if install_missing:
                t = access.satisfyVersion(
                                 name = dspec.name,
                     version_required = dspec.version_req,
                            available = target_hierarchy,
                         search_paths = search_dirs,
                    working_directory = targets_path,
                     update_installed = ('Update' if update_installed else None),
                                 type = 'target'
                )
            else:
                t = access.satisfyVersionFromSearchPaths(
                                 name = dspec.name,
                     version_required = dspec.version_req,
                         search_paths = search_dirs,
                                 type = 'target'
                )
        except access_common.Unavailable as e:
            errors.append(e)
        if not t:
            if install_missing:
                logger.error(
                    'could not install target %s for %s' %
                    (dspec, previous_name)
                )
            break
        else:
            target_hierarchy.append(t)
            previous_name = dspec.name
            assert(isinstance(t, Target))
            dspec = t.baseTargetSpec() #pylint: disable=no-member
            if not leaf_target:
                leaf_target = t
            if dspec is None:
                break
    if leaf_target is None:
        return (None, errors)
    # if we have a valid target, try to load the app-specific config data (if
    # any):
    app_config = {}
    if application_dir is not None:
        app_config_fname = os.path.join(application_dir, App_Config_File)
        if os.path.exists(app_config_fname):
            try:
                app_config = ordered_json.load(app_config_fname)
            except Exception as e:
                errors.append(Exception("Invalid application config.json: %s" % (e)))
    return (DerivedTarget(leaf_target, target_hierarchy[1:], app_config), errors)

class Target(pack.Pack):
    def __init__(self, path, installed_linked=False, latest_suitable_version=None):
        ''' Initialise a Target based on a directory. If the directory does not
            contain a valid target.json file the initialised object will test
            false, and will contain an error property containing the failure.
        '''
        # re-initialise with the information from the most-derived target
        super(Target, self).__init__(
                                      path,
               description_filename = Target_Description_File,
                   installed_linked = installed_linked,
                    schema_filename = Schema_File,
            latest_suitable_version = latest_suitable_version
        )

    def baseTargetSpec(self):
        ''' returns pack.DependencySpec for the base target of this target (or
            None if this target does not inherit from another target.
        '''
        inherits = self.description.get('inherits', {})
        if len(inherits) == 1:
            return pack.DependencySpec(list(inherits.items())[0][0], list(inherits.items())[0][1])
        elif len(inherits) > 1:
            logger.error('target %s specifies multiple base targets, but only one is allowed', self.getName())
        return None

    def getRegistryNamespace(self):
        return Registry_Namespace

    def getConfig(self):
        return self.description.get('config', OrderedDict())

class DerivedTarget(Target):
    def __init__(self, leaf_target, base_targets, app_config):
        ''' Initialise a DerivedTarget (representing an inheritance hierarchy of
            Targets.), given the most-derived Target description, and a set of
            available Targets to compose the rest of the lineage from.

            DerivedTarget provides build & debug commands, and access to the
            derived target config info (merged with the application config
            info from config.json, if any).

            It's possible to update the application config for an existing
            DerivedTarget instance.

            DerivedTarget can also be used as a stand-in for the most-derived
            (leaf) target in the inheritance hierarchy.
        '''

        # initialise the base class as a copy of leaf_target
        super(DerivedTarget, self).__init__(
                               path = leaf_target.path,
                   installed_linked = leaf_target.installed_linked,
            latest_suitable_version = leaf_target.latest_suitable_version
        )

        self.hierarchy = [leaf_target] + base_targets[:]
        self.config = None
        self.app_config = app_config


    # override truthiness to test validity of the entire hierarchy:
    def __nonzero__(self):
        for t in self.hierarchy:
            if not t: return False
        return bool(len(self.hierarchy))
    def __bool__(self):
        return self.__nonzero__()

    def getScript(self, scriptname):
        ''' return the specified script if one exists (possibly inherited from
            a base target)
        '''
        for t in self.hierarchy:
            if 'scripts' in t.description and scriptname in t.description['scripts']:
                return t.description['scripts'][scriptname]
        return None

    def _loadConfig(self):
        ''' load the configuration information from the target hierarchy '''
        config_dicts = [self.app_config] + [t.getConfig() for t in self.hierarchy]
        self.config = _mergeDictionaries(*config_dicts)
        # note that backwards compatibility with the "similarTo" data that used
        # to be used for target-dependencies is ensured at the point of use. We
        # don't merge similarTo into the config because it might break things.

    def _ensureConfig(self):
        if self.config is None:
            self._loadConfig()

    def setApplicationConfig(self, config):
        ''' set the application-config data to the contents of the
            dictionary-like object `config`
        '''
        self.app_config = config
        self._loadConfig()

    def getConfigValue(self, conf_key):
        self._ensureConfig()
        # jsonpointer, pip install jsonpointer, BSD 3 Clause
        import jsonpointer
        try:
            return jsonpointer.resolve_pointer(self.config, conf_key)
        except jsonpointer.JsonPointerException as e:
            # fall back to legacy dot-separated pointers
            key_path = conf_key.split('.');
            c = self.config
            for part in key_path:
                if part in c:
                    c = c[part]
                else:
                    return None
            return c

    def getSimilarTo_Deprecated(self):
        r = []
        for t in self.hierarchy:
            r.append(t.getName())
            r += t.description.get('similarTo', [])
        return r

    def getMergedConfig(self):
        self._ensureConfig()
        return self.config

    def getToolchainFiles(self):
        ''' return a list of toolchain file paths in override order (starting
            at the bottom/leaf of the hierarchy and ending at the base)
        '''
        return [
            os.path.join(x.path, x.description['toolchain']) for x in self.hierarchy if 'toolchain' in x.description
        ]

    @classmethod
    def addBuildOptions(cls, parser):
        parser.add_argument('-G', '--cmake-generator', dest='cmake_generator',
           default='Ninja',
           choices=(
               'Unix Makefiles',
               'Ninja',
               'Xcode',
               'Sublime Text 2 - Ninja',
               'Sublime Text 2 - Unix Makefiles'
           )
        )

    @classmethod
    def _findNinja(cls):
        # sometimes ninja is called ninja-build
        for name in ('ninja', 'ninja-build'):
            if fsutils.which(name) is not None:
                return name
        # default to ninja:
        return 'ninja'

    @classmethod
    def overrideBuildCommand(cls, generator_name, targets=None):
        if targets is None:
            targets = []
        # when we build using cmake --build, the nice colourised output is lost
        # - so override with the actual build command for command-line
        # generators where people will care:
        try:
            r = {
                'Unix Makefiles': ['make'],
                'Ninja': [cls._findNinja()]
            }[generator_name]
            # all of the above build programs take the build targets (e.g.
            # "all") as the last arguments
            if targets is not None:
                r += targets
            return r
        except KeyError:
            return None

    def hintForCMakeGenerator(self, generator_name, component):
        try:
            name = self.getName()
            component_name = component.getName()
            return {
                'Xcode':
                    'to open the built project, run:\nopen ./build/%s/%s.xcodeproj' % (name, component_name),
                'Sublime Text 2 - Ninja':
                    'to open the built project, run:\nopen ./build/%s/%s.??' % (name, component_name),
                'Sublime Text 2 - Unix Makefiles':
                    'to open the built project, run:\nopen ./build/%s/%s.??' % (name, component_name)
            }[generator_name]
        except KeyError:
            return None

    def exec_helper(self, cmd, builddir):
        ''' Execute the given command, returning an error message if an error occured
            or None if the command was succesful.'''
        try:
            child = subprocess.Popen(cmd, cwd=builddir)
            child.wait()
        except OSError as e:
            if e.errno == errno.ENOENT:
                if cmd[0] == 'cmake':
                    return 'CMake is not installed, please follow the installation instructions at http://yottadocs.mbed.com/#installing'
                else:
                    return '%s is not installed' % (cmd[0])
            else:
                return 'command %s failed' % (cmd)
        if child.returncode:
            return 'command %s failed' % (cmd)

    @fsutils.dropRootPrivs
    def build(self, builddir, component, args, release_build=False, build_args=None, targets=None):
        ''' Execute the commands necessary to build this component, and all of
            its dependencies. '''
        if build_args is None:
            build_args = []
        if targets is None:
            targets = []
        # in the future this may be specified in the target description, but
        # for now we only support cmake, so everything is simple:
        build_type = ('Debug', 'RelWithDebInfo')[release_build]
        if build_type:
            cmd = ['cmake', '-D', 'CMAKE_BUILD_TYPE=%s' % build_type, '-G', args.cmake_generator, '.']
        else:
            cmd = ['cmake', '-G', args.cmake_generator, '.']
        res = self.exec_helper(cmd, builddir)
        if res is not None:
            return res
        # cmake error: the generated Ninja build file will not work on windows when arguments are read from
        # a file (@file) instead of the command line, since '\' in @file is interpreted as an escape sequence.
        # !!! FIXME: remove this once http://www.cmake.org/Bug/view.php?id=15278 is fixed!
        if args.cmake_generator == "Ninja" and os.name == 'nt':
            logger.debug("Converting back-slashes in build.ninja to forward-slashes")
            build_file = os.path.join(builddir, "build.ninja")
            # We want to convert back-slashes to forward-slashes, except in macro definitions, such as
            # -DYOTTA_COMPONENT_VERSION = \"0.0.1\". So we use a little trick: first we change all \"
            # strings to an unprintable ASCII char that can't appear in build.ninja (in this case \1),
            # then we convert all the back-slashed to forward-slashes, then we convert '\1' back to \".
            try:
                f = open(build_file, "r+t")
                data = f.read()
                data = data.replace('\\"', '\1')
                data = data.replace('\\', '/')
                data = data.replace('\1', '\\"')
                f.seek(0)
                f.write(data)
                f.close()
            except:
                return 'Unable to update "%s", aborting' % build_file
        build_command = self.overrideBuildCommand(args.cmake_generator, targets=targets)
        if build_command:
            cmd = build_command + build_args
        else:
            cmd = ['cmake', '--build', builddir]
            if len(targets):
                # !!! FIXME: support multiple targets with the default CMake
                # build command
                cmd += ['--target', targets[0]]
            cmd += build_args
        res = self.exec_helper(cmd, builddir)
        if res is not None:
            return res
        hint = self.hintForCMakeGenerator(args.cmake_generator, component)
        if hint:
            logger.info(hint)

    def findProgram(self, builddir, program):
        ''' Return the builddir-relative path of program, if only a partial
            path is specified. Returns None and logs an error message if the
            program is ambiguous or not found
	'''
        # if this is an exact match, do no further checking:
        if os.path.isfile(os.path.join(builddir, program)):
            logging.info('found %s' % program)
            return program
        exact_matches = []
        insensitive_matches = []
        approx_matches = []
        for path, dirs, files in os.walk(builddir):
            if program in files:
                exact_matches.append(os.path.relpath(os.path.join(path, program), builddir))
                continue
            files_lower = [f.lower() for f in files]
            if program.lower() in files_lower:
                insensitive_matches.append(
                    os.path.relpath(
                        os.path.join(path, files[files_lower.index(program.lower())]),
                        builddir
                    )
                )
                continue
            # !!! TODO: in the future add approximate string matching (typos,
            # etc.), for now we just test stripping any paths off program, and
            # looking for substring matches:
            pg_basen_lower_noext = os.path.splitext(os.path.basename(program).lower())[0]
            for f in files_lower:
                if pg_basen_lower_noext in f:
                    approx_matches.append(
                        os.path.relpath(
                            os.path.join(path, files[files_lower.index(f)]),
                            builddir
                        )
                    )

        if len(exact_matches) == 1:
            logging.info('found %s at %s', program, exact_matches[0])
            return exact_matches[0]
        elif len(exact_matches) > 1:
            logging.error(
                '%s matches multiple executables, please use a full path (one of %s)' % (
                    program,
                    ', or '.join(['"'+os.path.join(m, program)+'"' for m in exact_matches])
                )
            )
            return None
        # if we have matches with and without a file extension, prefer the
        # no-file extension version, and discard the others (so we avoid
        # picking up post-processed files):
        reduced_approx_matches = []
        for m in approx_matches:
            root = os.path.splitext(m)[0]
            if (m == root) or (root not in approx_matches):
                reduced_approx_matches.append(m)
        approx_matches = reduced_approx_matches

        for matches in (insensitive_matches, approx_matches):
            if len(matches) == 1:
                logging.info('found %s at %s' % (
                    program, matches[0]
                ))
                return matches[0]
            elif len(matches) > 1:
                logging.error(
                    '%s is similar to several executables found. Please use an exact name:\n%s' % (
                        program,
                        '\n'.join(matches)
                    )
                )
                return None
        logging.error('could not find program "%s" to debug' %  program)
        return None

    def debug(self, builddir, program):
        ''' Launch a debugger for the specified program. Uses the `debug`
            script if specified by the target, falls back to the `debug` and
            `debugServer` commands if not. `program` is inserted into the
            $program variable in commands.
        '''
        try:
            signal.signal(signal.SIGINT, _ignoreSignal);
            if self.getScript('debug') is not None:
                return self._debugWithScript(builddir, program)
            elif 'debug' in self.description:
                logger.warning(
                    'target %s provides deprecated debug property. It should '+
                    'provide script.debug instead.', self.getName()

                )
                return self._debugDeprecated(builddir, program)
            else:
                return "Target %s does not specify debug commands" % self
        finally:
            # clear the sigint handler
            signal.signal(signal.SIGINT, signal.SIG_DFL);

    @fsutils.dropRootPrivs
    def _debugWithScript(self, builddir, program):
        child = None
        try:
            prog_path = prog_path = self.findProgram(builddir, program)
            if prog_path is None:
                return

            cmd = [
                os.path.expandvars(string.Template(x).safe_substitute(program=prog_path))
                for x in self.getScript('debug')
            ]
            logger.debug('starting debugger: %s', cmd)
            child = subprocess.Popen(
                cmd, cwd = builddir
            )
            child.wait()
            if child.returncode:
                return "debug process exited with status %s" % child.returncode
            child = None
        except:
            # reset the terminal, in case the debugger has screwed it up
            os.system('reset')
            raise
        finally:
            if child is not None:
                try:
                    child.terminate()
                except OSError as e:
                    pass

    @fsutils.dropRootPrivs
    def _debugDeprecated(self, builddir, program):
        prog_path = self.findProgram(builddir, program)
        if prog_path is None:
            return

        with open(os.devnull, "w") as dev_null:
            daemon = None
            child = None
            try:
                # debug-server is the old name, debugServer is the new name
                debug_server_prop = 'debugServer'
                if not debug_server_prop in self.description:
                    debug_server_prop = 'debug-server'

                if debug_server_prop in self.description:
                    logger.debug('starting debug server...')
                    daemon = subprocess.Popen(
                                   self.description[debug_server_prop],
                             cwd = builddir,
                          stdout = dev_null,
                          stderr = dev_null,
                      preexec_fn = _newPGroup
                    )
                else:
                    daemon = None

                cmd = [
                    os.path.expandvars(string.Template(x).safe_substitute(program=prog_path))
                    for x in self.description['debug']
                ]
                logger.debug('starting debugger: %s', cmd)
                child = subprocess.Popen(
                    cmd, cwd = builddir
                )
                child.wait()
                if child.returncode:
                    return "debug process executed with status %s" % child.returncode
                child = None
            except:
                # reset the terminal, in case the debugger has screwed it up
                os.system('reset')
                raise
            finally:
                if child is not None:
                    try:
                        child.terminate()
                    except OSError as e:
                        pass
                if daemon is not None:
                    logger.debug('shutting down debug server...')
                    try:
                        daemon.terminate()
                    except OSError as e:
                        pass

    @fsutils.dropRootPrivs
    def test(self, cwd, test_command, filter_command, forward_args):
        # we assume that test commands are relative to the current directory.
        test_command = './' + test_command
        test_script = self.getScript('test')
        if test_script is None:
            cmd = shlex.split(test_command)
        else:
            cmd = [
                os.path.expandvars(string.Template(x).safe_substitute(program=test_command))
                for x in test_script
            ] + forward_args

        test_child = None
        test_filter = None
        try:
            logger.debug('running test: %s', cmd)
            if filter_command:
                logger.debug('using output filter command: %s', filter_command)
                test_child = subprocess.Popen(
                    cmd, cwd = cwd, stdout = subprocess.PIPE
                )
                try:
                    test_filter = subprocess.Popen(
                        filter_command, cwd = cwd, stdin = test_child.stdout
                    )
                except OSError as e:
                    logger.error('error starting test output filter "%s": %s', filter_command, e)
                    test_child.terminate()
                    return 1
                test_filter.communicate()
                test_child.terminate()
                test_child.stdout.close()
                returncode = test_filter.returncode
                test_child = None
                test_filter = None
                if returncode:
                    logger.debug("test filter exited with status %s (=fail)", returncode)
                    return 1
            else:
                try:
                    test_child = subprocess.Popen(
                        cmd, cwd = cwd
                    )
                except OSError as e:
                    if e.errno == errno.ENOENT:
                        logger.error('Error: no such file or directory: "%s"', cmd[0])
                        return 1
                    raise
                test_child.wait()
                returncode = test_child.returncode
                test_child = None
                if returncode:
                    logger.debug("test process exited with status %s (=fail)", returncode)
                    return 1
        finally:
            if test_child is not None:
                test_child.terminate()
            if test_filter is not None:
                test_filter.terminate()
        logger.debug("test %s passed", test_command)
        return 0



#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import os
import tempfile
import copy

# internal modules:
from yotta.lib.fsutils import mkDirP, rmRf
from yotta.lib.detect import systemDefaultTarget
from . import cli


Test_Tests = {
'module.json':'''{
  "name": "test-tests",
  "version": "0.0.0",
  "description": "Test yotta's compilation of tests.",
  "author": "James Crosby <james.crosby@arm.com>",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ]
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}''',
'test-tests/foo.h':'int foo();',
'test/a/bar.c':'#include "test-tests/foo.h"\nint main(){ foo(); return 0; }',
'test/b/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/b/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/c/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/c/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/d/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/d/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/e/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/e/b/a/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/f/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/f/a/b/a/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }',
'test/g/a/a/a/bar.c':'#include "test-tests/foo.h"\nint bar(); int main(){ foo(); bar(); return 0; }',
'test/g/a/a/b/bar.c':'#include "stdio.h"\nint bar(){ printf("bar!\\n"); return 7; }'
}

Test_Fitler_Pass = copy.copy(Test_Tests)
Test_Fitler_Pass['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "grep",
      "!"
    ]
  }
}'''

Test_Fitler_Fail = copy.copy(Test_Tests)
Test_Fitler_Fail['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "grep",
      "string that isnt in the output"
    ]
  }
}'''

Test_Fitler_NotFound = copy.copy(Test_Tests)
Test_Fitler_NotFound['module.json'] = '''{
  "name": "test-tests",
  "version": "0.0.0",
  "licenses": [
    {
      "url": "https://spdx.org/licenses/Apache-2.0",
      "type": "Apache-2.0"
    }
  ],
  "scripts": {
    "testReporter": [
      "commandthatshouldntexist"
    ]
  }
}'''

def isWindows():
    return os.name == 'nt'

class TestCLITest(unittest.TestCase):
    def writeTestFiles(self, files, add_space_in_path=False):
        test_dir = tempfile.mkdtemp()
        if add_space_in_path:
            test_dir = test_dir + ' spaces in path'

        for path, contents in files.items():
            path_dir, file_name =  os.path.split(path)
            path_dir = os.path.join(test_dir, path_dir)
            mkDirP(path_dir)
            with open(os.path.join(path_dir, file_name), 'w') as f:
                f.write(contents)
        return test_dir

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_tests(self):
        test_dir = self.writeTestFiles(Test_Tests, True)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'build'], test_dir)
        output = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        self.assertIn('test-a passed', output)
        self.assertIn('test-c passed', output)
        self.assertIn('test-d passed', output)
        self.assertIn('test-e passed', output)
        self.assertIn('test-f passed', output)
        self.assertIn('test-g passed', output)
        rmRf(test_dir)

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_testOutputFilterPassing(self):
        test_dir = self.writeTestFiles(Test_Fitler_Pass, True)
        stdout = self.runCheckCommand(['--target', systemDefaultTarget(), 'test'], test_dir)
        rmRf(test_dir)

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_testOutputFilterFailing(self):
        test_dir = self.writeTestFiles(Test_Fitler_Fail, True)
        stdout, stderr, statuscode = cli.run(['--target', systemDefaultTarget(), 'test'], cwd=test_dir)
        if statuscode == 0:
            print(stdout)
            print(stderr)
        self.assertIn('test-a failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-c failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-d failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-e failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-f failed', '%s %s' % (stdout, stderr))
        self.assertIn('test-g failed', '%s %s' % (stdout, stderr))
        self.assertNotEqual(statuscode, 0)
        rmRf(test_dir)

    @unittest.skipIf(isWindows(), "can't build natively on windows yet")
    def test_testOutputFilterNotFound(self):
        test_dir = self.writeTestFiles(Test_Fitler_NotFound, True)
        stdout, stderr, statuscode = cli.run(['--target', systemDefaultTarget(), 'test'], cwd=test_dir)
        if statuscode == 0:
            print(stdout)
            print(stderr)
        self.assertNotEqual(statuscode, 0)
        rmRf(test_dir)

    def runCheckCommand(self, args, test_dir):
        stdout, stderr, statuscode = cli.run(args, cwd=test_dir)
        if statuscode != 0:
            print('command failed with status %s' % statuscode)
            print(stdout)
            print(stderr)
        self.assertEqual(statuscode, 0)
        return '%s %s' % (stdout, stderr)



# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging

# validate, , validate things, internal
from .lib import validate

def addOptions(parser):
    parser.add_argument('component', default=None, nargs='?',
        help='If specified, update (and if necessary install) this module '+
             'instead of updating the dependencies of the current module.'
    )
    parser.add_argument('-l', '--update-linked', dest='update_linked',
        action='store_true', default=False,
        help='Traverse linked modules, and update dependencies found there too.'
    )
    parser.add_argument('--test-dependencies', dest='update_test_deps',
        choices=('none', 'all', 'own'), default='own',
        help='Control the update of dependencies necessary for building tests.'
    )

def execCommand(args, following_args):
    c = validate.currentDirectoryModule()
    if not c:
        return 1
    logging.debug('update for %s' % c)
    if not args.target:
        logging.error('No target has been set, use "yotta target" to set one.')
        return 1
    target, errors = c.satisfyTarget(args.target, update_installed=True)
    if errors:
        for error in errors:
            logging.error(error)
        return 1
    if args.component is None:
        updateDeps(c, target, args)
    else:
        updateComponent(c, target, args)


def updateDeps(c, target, args):
    update_test_deps = {
         'own': 'toplevel',
         'all': True,
        'none': False
    }[args.update_test_deps]

    components, errors = c.satisfyDependenciesRecursive(
                          target = target,
                update_installed = True,
                  traverse_links = args.update_linked,
            available_components = [(c.getName(), c)],
                            test = update_test_deps
        )
    status = 0
    for error in errors:
        status = 1
        logging.error(error)
    return status


def updateComponent(c, target, args):
    # find the specified component in existing dependencies:
    update_test_deps = {
         'own': 'toplevel',
         'all': True,
        'none': False
    }[args.update_test_deps]

    dependencies = c.getDependenciesRecursive(
                      target = target,
        available_components = [(c.getName(), c)],
                        test = update_test_deps
    )
    if not args.component in dependencies:
        logging.error('%s is not a dependency of this module. Maybe you meant to "yotta install" it?', args.component)
        return 1
    #if not dependencies[args.component]:
    #    logging.error('%s is not valid: %s', args.component, dependencies[args.component].getError())
    #    return 1

    update_dependencies = dependencies[args.component].getDependenciesRecursive(
                      target = target,
        available_components = [(c.getName(), c)] + list(dependencies.items()),
                        test = update_test_deps
    )

    components, errors = c.satisfyDependenciesRecursive(
                      target = target,
            update_installed = set(list(update_dependencies.keys()) + [args.component]),
              traverse_links = args.update_linked,
        available_components = [(c.getName(), c)],
                        test = update_test_deps
    )

    status = 0
    for error in errors:
        status = 1
        logging.error(error)
    return status




# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# this module exists to share global config amongst modules

conf = {}

def set(name, value):
    global conf
    conf[name] = value

def get(name):
    global conf
    return conf[name]




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import logging
from collections import OrderedDict

# access, , get components, internal
import access
import access_common
# pool, , shared thread pool, internal
#from pool import pool
# vcs, , represent version controlled directories, internal
import vcs
# fsutils, , misc filesystem utils, internal
import fsutils
# Pack, , common parts of Components/Targets, internal
import pack
# Target, , represent an installed target, internal
import target
# sourceparse, , parse version source urls, internal
import sourceparse

# !!! FIXME: should components lock their description file while they exist?
# If not there are race conditions where the description file is modified by
# another process (or in the worst case replaced by a symlink) after it has
# been opened and before it is re-written


# Constants
Modules_Folder = 'yotta_modules'
Targets_Folder = 'yotta_targets'
Component_Description_File = 'module.json'
Component_Description_File_Fallback = 'package.json'
Registry_Namespace = 'modules'
Schema_File = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'schema', 'module.json')

logger = logging.getLogger('components')
VVVERBOSE_DEBUG = logging.DEBUG - 8

def _truthyConfValue(v):
    ''' Determine yotta-config truthiness. In yotta config land truthiness is
        different to python or json truthiness (in order to map nicely only
        preprocessor and CMake definediness):

          json      -> python -> truthy/falsey
          false     -> False  -> Falsey
          null      -> None   -> Falsey
          undefined -> None   -> Falsey
          0         -> 0      -> Falsey
          ""        -> ""     -> Truthy (different from python)
          "0"       -> "0"    -> Truthy
          {}        -> {}     -> Truthy (different from python)
          []        -> []     -> Truthy (different from python)
          everything else is truthy
    '''
    if v is False:
        return False
    elif v is None:
        return False
    elif v == 0:
        return False
    else:
        # everything else is truthy!
        return True

# API
class Component(pack.Pack):
    def __init__(self, path, installed_linked=False, latest_suitable_version=None, test_dependency=False):
        ''' How to use a Component:

            Initialise it with the directory into which the component has been
            downloaded, (or with a symlink that points to a directory
            containing the component)

            Check that 'if component:' is true, which indicates that the
            download is indeed a valid component.

            Check that component.getVersion() returns the version you think
            you've downloaded.

            Use component.getDependencySpecs() to get the names of the
            dependencies of the component, or component.getDependencies() to
            get Component objects (which may not be valid unless the
            dependencies have been installed) for each of the dependencies.

        '''
        logger.log(VVVERBOSE_DEBUG, "Component: " +  path +  ' installed_linked=' + str(installed_linked))
        warn_deprecated_filename = False
        if (not os.path.exists(os.path.join(path, Component_Description_File))) and \
           os.path.exists(os.path.join(path, Component_Description_File_Fallback)):
            warn_deprecated_filename = True
            description_filename = Component_Description_File_Fallback
        else:
            description_filename = Component_Description_File
        super(Component, self).__init__(
                                      path,
               description_filename = description_filename,
                   installed_linked = installed_linked,
                    schema_filename = Schema_File,
            latest_suitable_version = latest_suitable_version
        )
        if warn_deprecated_filename:
            logger.warning(
                "Component %s uses deprecated %s file, use %s instead." % (
                    self.getName(),
                    Component_Description_File_Fallback,
                    Component_Description_File
                )
            )
        self.installed_dependencies = False
        self.dependencies_failed = False
        self.is_test_dependency = test_dependency

    def getDependencySpecs(self, target=None):
        ''' Returns [DependencySpec]

            These are returned in the order that they are listed in the
            component description file: this is so that dependency resolution
            proceeds in a predictable way.
        '''
        deps = []

        deps += [pack.DependencySpec(x[0], x[1], False) for x in self.description.get('dependencies', {}).items()]
        target_deps = self.description.get('targetDependencies', {})
        if target is not None:
            for conf_key, target_conf_deps in target_deps.items():
                if _truthyConfValue(target.getConfigValue(conf_key)) or conf_key in target.getSimilarTo_Deprecated():
                    logger.debug(
                        'Adding target-dependent dependency specs for target config %s to component %s' %
                        (conf_key, self.getName())
                    )
                    deps += [pack.DependencySpec(x[0], x[1], False) for x in target_conf_deps.items()]


        deps += [pack.DependencySpec(x[0], x[1], True) for x in self.description.get('testDependencies', {}).items()]
        target_deps = self.description.get('testTargetDependencies', {})
        if target is not None:
            for conf_key, target_conf_deps in target_deps.items():
                if _truthyConfValue(target.getConfigValue(conf_key)) or conf_key in target.getSimilarTo_Deprecated():
                    logger.debug(
                        'Adding test-target-dependent dependency specs for target config %s to component %s' %
                        (conf_key, self.getName())
                    )
                    deps += [pack.DependencySpec(x[0], x[1], True) for x in target_conf_deps.items()]

        # remove duplicates (use the first occurrence)
        seen = set()
        r = []
        for dep in deps:
            if not dep.name in seen:
                r.append(dep)
                seen.add(dep.name)

        return r

    def hasDependency(self, name, target=None):
        ''' Check if this module has any dependencies with the specified name
            in its dependencies list, or in target dependencies for the
            specified target
        '''
        if name in self.description.get('dependencies', {}).keys():
            return True

        target_deps = self.description.get('targetDependencies', {})
        if target is not None:
            for conf_key, target_conf_deps in target_deps.items():
                if _truthyConfValue(target.getConfigValue(conf_key)) or conf_key in target.getSimilarTo_Deprecated():
                    if name in target_conf_deps:
                        return True
        return False


    def getDependencies(self,
        available_components = None,
                 search_dirs = None,
                      target = None,
              available_only = False,
                        test = False,
                    warnings = True
        ):
        ''' Returns {component_name:component}
        '''
        if search_dirs is None:
            search_dirs = [self.modulesPath()]
        available_components = self.ensureOrderedDict(available_components)

        components, errors = self.__getDependenciesWithProvider(
            available_components = available_components,
                     search_dirs = search_dirs,
                          target = target,
                update_installed = False,
                        provider = self.provideInstalled,
                            test = test
        )
        if warnings:
            for error in errors:
                logger.warning(error)
        if available_only:
            components = OrderedDict((k, v) for k, v in components.items() if v)
        return components

    def __getDependenciesWithProvider(self,
                      available_components = None,
                               search_dirs = None,
                                    target = None,
                          update_installed = False,
                                  provider = None,
                                      test = False
   ):
        ''' Get installed components using "provider" to find (and possibly
            install) components.

            See documentation for __getDependenciesRecursiveWithProvider

            returns (components, errors)
        '''
        errors = []
        modules_path = self.modulesPath()
        def satisfyDep(dspec):
            try:
                r = provider(
                  dspec,
                  available_components,
                  search_dirs,
                  modules_path,
                  update_installed,
                  self.getName()
                )
                if r and not sourceparse.parseSourceURL(dspec.version_req).semanticSpecMatches(r.getVersion()):
                    logger.debug('%s does not meet specification %s required by %s' % (r.getName(), dspec.version_req, self.getName()))
                    r.setError('does not meet specification %s required by %s' % (dspec.version_req, self.getName()))
                return r
            except access_common.Unavailable as e:
                errors.append(e)
                self.dependencies_failed = True
            except vcs.VCSError as e:
                errors.append(e)
                self.dependencies_failed = True
        specs = self.getDependencySpecs(target=target)
        if not test:
            # filter out things that aren't test dependencies if necessary:
            specs = [x for x in specs if not x.is_test_dependency]
        #dependencies = pool.map(
        dependencies = map(
            satisfyDep, specs
        )
        self.installed_dependencies = True
        # stable order is important!
        return (OrderedDict([((d and d.getName()) or specs[i].name, d) for i, d in enumerate(dependencies)]), errors)


    def __getDependenciesRecursiveWithProvider(self,
                               available_components = None,
                                        search_dirs = None,
                                             target = None,
                                     traverse_links = False,
                                   update_installed = False,
                                           provider = None,
                                               test = False,
                                         _processed = None
    ):
        ''' Get installed components using "provider" to find (and possibly
            install) components.

            This function is called with different provider functions in order
            to retrieve a list of all of the dependencies, or install all
            dependencies.

            Returns
            =======
                (components, errors)

                components: dictionary of name:Component
                errors: sequence of errors

            Parameters
            ==========
                available_components:
                    None (default) or a dictionary of name:component. This is
                    searched before searching directories or fetching remote
                    components

                search_dirs:
                    None (default), or sequence of directories to search for
                    already installed, (but not yet loaded) components. Used so
                    that manually installed or linked components higher up the
                    dependency tree are found by their users lower down.

                    These directories are searched in order, and finally the
                    current directory is checked.

                target:
                    None (default), or a Target object. If specified the target
                    name and it's similarTo list will be used in resolving
                    dependencies. If None, then only target-independent
                    dependencies will be installed

                traverse_links:
                    False (default) or True: whether to recurse into linked
                    dependencies. You normally want to set this to "True" when
                    getting a list of dependencies, and False when installing
                    them (unless the user has explicitly asked dependencies to
                    be installed in linked components).

                provider: None (default) or function:
                          provider(
                            dependency_spec,
                            available_components,
                            search_dirs,
                            working_directory,
                            update_if_installed
                          )
                test:
                    True, False, 'toplevel': should test-only dependencies be
                    included (yes, no, or only at this level, not recursively)
        '''
        def recursionFilter(c):
            if not c:
                logger.debug('do not recurse into failed component')
                # don't recurse into failed components
                return False
            if c.getName() in _processed:
                logger.debug('do not recurse into already processed component: %s' % c)
                return False
            if c.installedLinked() and not traverse_links:
                return False
            return True
        available_components = self.ensureOrderedDict(available_components)
        if search_dirs is None:
            search_dirs = []
        if _processed is None:
            _processed = set()
        assert(test in [True, False, 'toplevel'])
        search_dirs.append(self.modulesPath())
        logger.debug('process %s\nsearch dirs:%s' % (self.getName(), search_dirs))
        if self.isTestDependency():
            logger.debug("won't provide test dependencies recursively for test dependency %s", self.getName())
            test = False
        components, errors = self.__getDependenciesWithProvider(
            available_components = available_components,
                     search_dirs = search_dirs,
                update_installed = update_installed,
                          target = target,
                        provider = provider,
                            test = test
        )
        _processed.add(self.getName())
        if errors:
            errors = ['Failed to satisfy dependencies of %s:' % self.path] + errors
        need_recursion = [x for x in filter(recursionFilter, components.values())]
        available_components.update(components)
        logger.debug('processed %s\nneed recursion: %s\navailable:%s\nsearch dirs:%s' % (self.getName(), need_recursion, available_components, search_dirs))
        if test == 'toplevel':
            test = False
        # NB: can't perform this step in parallel, since the available
        # components list must be updated in order
        for c in need_recursion:
            dep_components, dep_errors = c.__getDependenciesRecursiveWithProvider(
                available_components = available_components,
                         search_dirs = search_dirs,
                              target = target,
                      traverse_links = traverse_links,
                    update_installed = update_installed,
                            provider = provider,
                                test = test,
                          _processed = _processed
            )
            available_components.update(dep_components)
            components.update(dep_components)
            errors += dep_errors
        return (components, errors)

    def provideInstalled(self,
                        dspec,
         available_components,
                  search_dirs,
            working_directory,
             update_installed,
                  dep_of_name
        ):
        r = access.satisfyFromAvailable(dspec.name, available_components)
        if r:
            if r.isTestDependency() and not dspec.is_test_dependency:
                logger.debug('test dependency subsequently occurred as real dependency: %s', r.getName())
                r.setTestDependency(False)
            return r
        update_if_installed = False
        if update_installed is True:
            update_if_installed = True
        elif update_installed:
            update_if_installed = dspec.name in update_installed
        r = access.satisfyVersionFromSearchPaths(dspec.name, dspec.version_req, search_dirs, update_if_installed)
        if r:
            r.setTestDependency(dspec.is_test_dependency)
            return r
        # return a module initialised to the path where we would have
        # installed this module, so that it's possible to use
        # getDependenciesRecursive to find a list of failed dependencies,
        # as well as just available ones
        # note that this Component object may still be valid (usable to
        # attempt a build), if a different version was previously installed
        # on disk at this location (which means we need to check if the
        # existing version is linked)
        default_path = os.path.join(self.modulesPath(), dspec.name)
        r = Component(
                               default_path,
             test_dependency = dspec.is_test_dependency,
            installed_linked = fsutils.isLink(default_path)
        )
        return r

    def getDependenciesRecursive(self,
                 available_components = None,
                            processed = None,
                          search_dirs = None,
                               target = None,
                       available_only = False,
                                 test = False
        ):
        ''' Get available and already installed components, don't check for
            remotely available components. See also
            satisfyDependenciesRecursive()

            Returns {component_name:component}
        '''
        components, errors = self.__getDependenciesRecursiveWithProvider(
           available_components = available_components,
                    search_dirs = search_dirs,
                         target = target,
                 traverse_links = True,
               update_installed = False,
                       provider = self.provideInstalled,
                           test = test
        )
        for error in errors:
            logger.error(error)
        if available_only:
            components = OrderedDict((k, v) for k, v in components.items() if v)
        return components

    def modulesPath(self):
        return os.path.join(self.path, Modules_Folder)

    def targetsPath(self):
        return os.path.join(self.path, Targets_Folder)

    def satisfyDependenciesRecursive(
                            self,
            available_components = None,
                     search_dirs = None,
                update_installed = False,
                  traverse_links = False,
                          target = None,
                            test = False
        ):
        ''' Retrieve and install all the dependencies of this component and its
            dependencies, recursively, or satisfy them from a collection of
            available_components or from disk.

            Returns
            =======
                (components, errors)

                components: dictionary of name:Component
                errors: sequence of errors

            Parameters
            ==========

                available_components:
                    None (default) or a dictionary of name:component. This is
                    searched before searching directories or fetching remote
                    components

                search_dirs:
                    None (default), or sequence of directories to search for
                    already installed, (but not yet loaded) components. Used so
                    that manually installed or linked components higher up the
                    dependency tree are found by their users lower down.

                    These directories are searched in order, and finally the
                    current directory is checked.

                update_installed:
                    False (default), True, or set(): whether to check the
                    available versions of installed components, and update if a
                    newer version is available. If this is a set(), only update
                    things in the specified set.

                traverse_links:
                    False (default) or True: whether to recurse into linked
                    dependencies when updating/installing.

                target:
                    None (default), or a Target object. If specified the target
                    name and it's similarTo list will be used in resolving
                    dependencies. If None, then only target-independent
                    dependencies will be installed

                test:
                    True, False, or 'toplevel: should test-only dependencies be
                    installed? (yes, no, or only for this module, not its
                    dependencies).

        '''
        def provider(
            dspec,
            available_components,
            search_dirs,
            working_directory,
            update_installed,
            dep_of_name=None
        ):
            r = access.satisfyFromAvailable(dspec.name, available_components)
            if r:
                if r.isTestDependency() and not dspec.is_test_dependency:
                    logger.debug('test dependency subsequently occurred as real dependency: %s', r.getName())
                    r.setTestDependency(False)
                return r
            update_if_installed = False
            if update_installed is True:
                update_if_installed = True
            elif update_installed:
                update_if_installed = dspec.name in update_installed
            r = access.satisfyVersionFromSearchPaths(dspec.name, dspec.version_req, search_dirs, update_if_installed)
            if r:
                r.setTestDependency(dspec.is_test_dependency)
                return r
            # before resorting to install this module, check if we have an
            # existing linked module (which wasn't picked up because it didn't
            # match the version specification) - if we do, then we shouldn't
            # try to install, but should return that anyway:
            default_path = os.path.join(self.modulesPath(), dspec.name)
            if fsutils.isLink(default_path):
                r = Component(
                                       default_path,
                     test_dependency = dspec.is_test_dependency,
                    installed_linked = fsutils.isLink(default_path)
                )
                if r:
                    assert(r.installedLinked())
                    return r
                else:
                    logger.error('linked module %s is invalid: %s', dspec.name, r.getError())
                    return r

            r = access.satisfyVersionByInstalling(dspec.name, dspec.version_req, self.modulesPath())
            if not r:
                logger.error('could not install %s' % dspec.name)
            if r is not None:
                r.setTestDependency(dspec.is_test_dependency)
            return r

        return self.__getDependenciesRecursiveWithProvider(
           available_components = available_components,
                    search_dirs = search_dirs,
                         target = target,
                 traverse_links = traverse_links,
               update_installed = update_installed,
                       provider = provider,
                           test = test
        )

    def satisfyTarget(self, target_name_and_version, update_installed=False):
        ''' Ensure that the specified target name (and optionally version,
            github ref or URL) is installed in the targets directory of the
            current component
        '''
        application_dir = None
        if self.isApplication():
            application_dir = self.path
        return target.getDerivedTarget(
            target_name_and_version,
            self.targetsPath(),
            application_dir = application_dir,
            update_installed = update_installed
        )

    def installedDependencies(self):
        ''' Return true if satisfyDependencies has been called.

            Note that this is slightly different to when all of the
            dependencies are actually satisfied, but can be used as if it means
            that.
        '''
        return self.installed_dependencies

    def isApplication(self):
        ''' Return true if this module is an application instead of a reusable
            library '''
        return bool(len(self.getBinaries()))

    def getBinaries(self):
        ''' Return a dictionary of binaries to compile: {"dirname":"exename"},
            this is used when automatically generating CMakeLists '''
        # the module.json syntax is a subset of the package.json syntax: a
        # single string that defines the source directory to use to build an
        # executable with the same name as the component. This may be extended
        # to include the rest of the npm syntax in future (map of source-dir to
        # exe name).
        if 'bin' in self.description:
            return {self.description['bin']: self.getName()}
        else:
            return {}

    def licenses(self):
        ''' Return a list of licenses that apply to this module. (Strings,
            which may be SPDX identifiers)
        '''
        if 'license' in self.description:
            return [self.description['license']]
        else:
            return [x['type'] for x in self.description['licenses']]

    def getExtraIncludes(self):
        ''' Some components must export whole directories full of headers into
            the search path. This is really really bad, and they shouldn't do
            it, but support is provided as a concession to compatibility.
        '''
        if 'extraIncludes' in self.description:
            return [os.path.normpath(x) for x in self.description['extraIncludes']]
        else:
            return []

    def getExtraSysIncludes(self):
        ''' Some components (e.g. libc) must export directories of header files
            into the system include search path. They do this by adding a
            'extraSysIncludes' : [ array of directories ] field in their
            package description. This function returns the list of directories
            (or an empty list), if it doesn't exist.
        '''
        if 'extraSysIncludes' in self.description:
            return [os.path.normpath(x) for x in self.description['extraSysIncludes']]
        else:
            return []

    def getRegistryNamespace(self):
        return Registry_Namespace

    def setTestDependency(self, status):
        self.is_test_dependency = status

    def isTestDependency(self):
        return self.is_test_dependency

    def __saveSpecForComponent(self, component):
        version = component.getVersion()
        if version.isTip():
            spec = '*'
        elif version.major() == 0:
            # for 0.x.x versions, when we save a dependency we don't use ^0.x.x
            # a that would peg to the exact version - instead we use ~ to peg
            # to the same minor version
            spec = '~' + str(version)
        else:
            spec = '^' + str(version)
        return spec

    def saveDependency(self, component, spec=None):
        if not 'dependencies' in self.description:
            self.description['dependencies'] = OrderedDict()
        if spec is None:
            spec = self.__saveSpecForComponent(component)
        self.description['dependencies'][component.getName()] = spec
        return spec

    def removeDependency(self, component):
        if not component in self.description.get('dependencies', {}):
            logger.error('%s is not listed as a dependency', component)
            return False
        del self.description['dependencies'][component]
        return True

    def getTestFilterCommand(self):
        ''' return the test-output filtering command (array of strings) that
            this module defines, if any. '''
        if 'scripts' in self.description and 'testReporter' in self.description['scripts']:
            return self.description['scripts']['testReporter']
        else:
            return None



# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging
import os
import threading
from collections import OrderedDict

# fsutils, , misc filesystem utils, internal
import fsutils
# Ordered JSON, , read & write json, internal
import ordered_json
# folders, , get places to install things, internal
import folders

#
# yotta's settings always written to ~/.yotta/config.json, but are read, in
# order from:
#
#  1. environment variables (YOTTA_{section_name.upper()}_{variable_name.upper()})
#  2. ~/.yotta/config.json
#  3. /usr/local/etc/yottaconfig.json
#  4. /etc/yottaconfig.json
#
# As soon as a value is found for a variable, the search is stopped.
#
#

# constants
user_config_file = os.path.join(folders.userSettingsDirectory(), 'config.json')
dir_config_file  = os.path.join('.','.yotta.json')

config_files = [
    dir_config_file,
    user_config_file,
]
if os.name == 'nt':
    config_files += [
        os.path.expanduser(os.path.join(folders.prefix(),'yotta.json'))
    ]
else:
    config_files += [
        os.path.expanduser(os.path.join(folders.prefix(),'etc','yotta.json')),
        os.path.join('etc','yotta.json')
    ]


# private state
parser = None
parser_lock = threading.Lock()

# private API

# class for reading JSON config files,
class _JSONConfigParser(object):
    def __init__(self):
        self.configs = OrderedDict()

    def read(self, filenames):
        '''' Read a list of files. Their configuration values are merged, with
             preference to values from files earlier in the list.
        '''
        for fn in filenames:
            try:
                self.configs[fn] = ordered_json.load(fn)
            except IOError:
                self.configs[fn] = OrderedDict()

    def get(self, path):
        ''' return a configuration value

            usage:
                get('section.property')

            Note that currently array indexes are not supported. You must
            get the whole array.

            returns None if any path element or the property is missing
        '''
        path = _splitPath([path])
        for config in self.configs.values():
            cur = config
            for el in path:
                if el in cur:
                    cur = cur[el]
                else:
                    cur = None
                    break
            if cur is not None:
                return cur
        return None

    def set(self, path, value=None, filename=None):
        ''' Set a configuration value. If no filename is specified, the
            property is set in the first configuration file. Note that if a
            filename is specified and the property path is present in an
            earlier filename then set property will be hidden.

            usage:
                set('section.property', value='somevalue')

            Note that currently array indexes are not supported. You must
            set the whole array.
        '''
        if filename is None:
            config = self._firstConfig()[1]
        else:
            config = self.configs[filename]

        path = _splitPath([path])
        for el in path[:-1]:
            if el in config:
                config = config[el]
            else:
                config[el] = OrderedDict()
                config = config[el]
        config[path[-1]] = value

    def write(self, filename=None):
        if filename is None:
            filename, data = self._firstConfig()
        elif filename in self.configs:
            data = self.configs[filename]
        else:
            raise ValueError('No such file.')
        dirname = os.path.dirname(filename)
        fsutils.mkDirP(dirname)
        ordered_json.dump(filename, data)

    def _firstConfig(self):
        for fn, data in self.configs.items():
            return fn, data
        raise ValueError('No configs available.')

def _splitPath(path):
    r = []
    for p in path:
        r += p.split('.')
    if not len(p):
        raise ValueError('A path must be specified.')
    return r

def _ensureParser():
    global parser
    with parser_lock:
        if not parser:
            parser = _JSONConfigParser()
            parser.read(config_files)

def _checkEnv(path):
    env_key = '_'.join(['YOTTA'] + [x.upper() for x in _splitPath(path)])
    try:
        return os.environ[env_key]
    except KeyError:
        return None

# public API

def get(path):
    value = _checkEnv(path)
    if value:
        logging.debug('read property from environment: %s', path)
        return value
    _ensureParser()
    with parser_lock:
        return parser.get(path)

def getProperty(section, name):
    return get(section + '.' + name)

def set(path, value, save_locally=False):
    if save_locally:
        filename = dir_config_file
    else:
        filename = user_config_file

    logging.debug('setProperty: %s %s:%s', path, type(value), value)
    _ensureParser()
    with parser_lock:
        parser.set(path, value=value, filename=filename)
        parser.write(filename)

def setProperty(section, name, value, save_locally=False):
    set(section+'.'+name, value, save_locally)




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# Utilities for parsing version source URLs, and determining how the
# specified version should be fetched.

# standard library modules, , ,
try:
    from urlparse import urlsplit
except ImportError:
    from urllib.parse import urlsplit #pylint: disable=no-name-in-module,import-error
import re

# version, , represent versions and specifications, internal
import version

class VersionSource(object):
    def __init__(self, source_type, location, spec):
        assert(source_type in ('registry', 'github', 'git', 'hg'))
        self.source_type = source_type
        self.location = location
        self.spec = spec
        try:
            self.semantic_spec = version.Spec(spec)
        except ValueError:
            # for git/github source URLs the spec is allowed to be a branch
            # name or tag name, as well as a valid semantic version
            # specification
            # !!! TODO: also allow hg here
            if source_type in ('git', 'github'):
                self.semantic_spec = None
            else:
                raise ValueError(
                    "Invalid semantic version spec: \"%s\"" % spec
                )

    def semanticSpec(self):
        return self.semantic_spec or version.Spec('*')

    def semanticSpecMatches(self, v):
        if self.semantic_spec is None:
            return True
        else:
            return self.semantic_spec.match(v)


def parseSourceURL(source_url):
    ''' Parse the specified version source URL (or version spec), and return an
        instance of VersionSource
    '''
    parsed = urlsplit(source_url)

    if '#' in source_url:
        without_fragment = source_url[:source_url.index('#')]
    else:
        without_fragment = source_url

    try:
        url_is_spec = version.Spec(source_url)
    except ValueError:
        url_is_spec = None

    if url_is_spec is not None:
        # if the url is an unadorned version specification (including an empty
        # string) then the source is the module registry:
        return VersionSource('registry', '', source_url)
    elif parsed.netloc.endswith('github.com'):
        # any URL onto github should be fetched over the github API, even if it
        # would parse as a valid git URL
        return VersionSource('github', parsed.path, parsed.fragment)
    elif parsed.scheme.startswith('git+') or parsed.path.endswith('.git'):
        # git+anything://anything or anything.git is a git repo:
        return VersionSource('git', without_fragment, parsed.fragment)
    elif parsed.scheme.startswith('hg+') or parsed.path.endswith('.hg'):
        # hg+anything://anything or anything.hg is a hg repo:
        return VersionSource('hg', without_fragment, parsed.fragment)
    elif re.match('^[a-z0-9_-]+/[a-z0-9_-]+$', without_fragment, re.I):
        # something/something#spec = github
        return VersionSource('github', without_fragment, parsed.fragment)

    # something/something@spec = github
    alternate_github_match = re.match('([a-z0-9_-]+/[a-z0-9_-]+) *@?([~^><=.0-9a-z\*-]*)', source_url, re.I)
    if alternate_github_match:
        return VersionSource('github', alternate_github_match.group(0), alternate_github_match.group(1))

    raise ValueError("Invalid version source url: \"%s\"" % (source_url))




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# we need circular imports, and circular imports of relative paths aren't
# supported in python 3, so unfortunately we have to do this:
import os
import sys
sys.path.insert(0, os.path.dirname(__file__))




# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import logging

# validate, , validate things, internal
from .lib import validate

def addOptions(parser):
    # no options
    pass

def execCommand(args, following_args):
    p = validate.currentDirectoryModuleOrTarget()
    if not p:
        return 1

    if not p.vcsIsClean():
        logging.error('The working directory is not clean. Commit before publishing!')
        return 1

    if p.description.get('bin', None) is not None:
        logging.warning(
            'This is an executable application, not a re-usable library module. Other modules will not be able to depend on it!'
        )
        # python 2 + 3 compatibility
        try:
            global input
            input = raw_input
        except NameError:
            pass
        raw_input("If you still want to publish it, press [enter] to continue.")

    error = p.publish(args.registry)
    if error:
        logging.error(error)
        return 1

    # tag the version published as 'latest'
    # !!! can't do this, as can't move tags in git?
    #p.commitVCS(tag='latest')
    logging.info('published latest version: %s', p.getVersion())
    return 0



#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import sys
import subprocess
import os

def run(arguments, cwd='.'):
    yottadir = os.path.join(os.path.dirname(os.path.realpath(__file__)), '..', '..')
    runyotta = [
        sys.executable,
        '-c',
        "import sys; sys.path.insert(0, '%s'); import yotta; yotta.main()" % yottadir
    ]
    child = subprocess.Popen(
          args = runyotta + arguments,
           cwd = cwd,
        stdout = subprocess.PIPE,
        stderr = subprocess.PIPE,
         stdin = subprocess.PIPE
    )
    out, err = child.communicate()
    return out.decode('utf-8'), err.decode('utf-8'), child.returncode






# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import tarfile
import logging
import os
import hashlib
import tempfile
import shutil
import functools
import random
import errno

# version, , represent versions and specifications, internal
import version
# fsutils, , misc filesystem utils, internal
import fsutils
# folders, , where yotta stores things, internal
import folders

logger = logging.getLogger('access')
cache_logger = logging.getLogger('cache')

# arbitrary default setting for download caching. Later maybe this can be a
# setting:
Max_Cached_Modules = 100

class AccessException(Exception):
    pass


class Unavailable(AccessException):
    pass


class TargetUnavailable(Unavailable):
    pass


class SpecificationNotMet(AccessException):
    pass


class RemoteVersion(version.Version):
    def __init__(self, version_string, url=None, name='unknown', friendly_source='unknown', friendly_version=None):
        self.name = name
        self.version_string = version_string
        self.friendly_version = friendly_version or version_string
        self.friendly_source = friendly_source
        super(RemoteVersion, self).__init__(version_string, url)

    def unpackInto(self, directory):
        raise NotImplementedError

    def __repr__(self):
        return u'%s@%s from %s' % (self.name, self.friendly_version, self.friendly_source)
    def __str__(self):
        return self.__unicode__()
    def __unicode__(self):
        return self.__repr__()

class RemoteComponent(object):

    @classmethod
    def createFromSource(cls, url, name=None):
        raise NotImplementedError

    def versionSpec(self):
        raise NotImplementedError

    def availableVersions(self):
        raise NotImplementedError

    def tipVersion(self):
        raise NotImplementedError

    @classmethod
    def remoteType(cls):
        raise NotImplementedError


def pruneCache():
    ''' Prune the cache '''
    cache_dir = folders.cacheDirectory()
    def fullpath(f):
        return os.path.join(cache_dir, f)
    # ensure cache exists
    fsutils.mkDirP(cache_dir)
    for f in sorted(
            [f for f in os.listdir(cache_dir) if os.path.isfile(fullpath(f))],
            key = lambda f: os.stat(fullpath(f)).st_mtime
        )[Max_Cached_Modules:]:
        cache_logger.debug('cleaning up cache file %s', f)
        fsutils.rmF(fullpath(f))
    cache_logger.debug('cache pruned to %s items', Max_Cached_Modules)

def sometimesPruneCache(p):
    ''' return decorator to prune cache after calling fn with a probability of p'''
    def decorator(fn):
        @functools.wraps(fn)
        def wrapped(*args, **kwargs):
            r = fn(*args, **kwargs)
            if random.random() < p:
                pruneCache()
            return r
        return wrapped
    return decorator

def unpackFrom(tar_file_path, to_directory):
    # first unpack into a sibling directory of the specified directory, and
    # then move it into place.

    # we expect our tarballs to contain a single top-level directory. We strip
    # off this name as we extract to minimise the path length

    into_parent_dir = os.path.dirname(to_directory)
    fsutils.mkDirP(into_parent_dir)
    temp_directory = tempfile.mkdtemp(dir=into_parent_dir)
    try:
        with tarfile.open(tar_file_path) as tf:
            strip_dirname = ''
            # get the extraction directory name from the first part of the
            # extraction paths: it should be the same for all members of
            # the archive
            for m in tf.getmembers():
                split_path = fsutils.fullySplitPath(m.name)
                logger.debug('process member: %s %s', m.name, split_path)
                if os.path.isabs(m.name) or '..' in split_path:
                    raise ValueError('archive uses invalid paths')
                if not strip_dirname:
                    if len(split_path) != 1 or not len(split_path[0]):
                        raise ValueError('archive does not appear to contain a single module')
                    strip_dirname = split_path[0]
                    continue
                else:
                    if split_path[0] != strip_dirname:
                        raise ValueError('archive does not appear to contain a single module')
                m.name = os.path.join(*split_path[1:])
                tf.extract(m, path=temp_directory)
        # make sure the destination directory doesn't exist:
        fsutils.rmRf(to_directory)
        shutil.move(temp_directory, to_directory)
        temp_directory = None
        logger.debug('extraction complete %s', to_directory)
    except IOError as e:
        if e.errno != errno.ENOENT:
            logger.error('failed to extract tarfile %s', e)
            fsutils.rmF(tar_file_path)
        raise
    finally:
        if temp_directory is not None:
            # if anything has failed, cleanup
            fsutils.rmRf(temp_directory)

def unpackFromCache(cache_key, to_directory):
    ''' If the specified cache key exists, unpack the tarball into the
        specified directory, otherwise raise KeyError.
    '''
    if cache_key is None:
        raise KeyError('"None" is never in cache')
    cache_dir = folders.cacheDirectory()
    fsutils.mkDirP(cache_dir)
    path = os.path.join(cache_dir, cache_key)
    logger.debug('attempt to unpack from cache %s -> %s', path, to_directory)
    try:
        unpackFrom(path, to_directory)
        cache_logger.debug('unpacked %s from cache into %s', cache_key, to_directory)
        return
    except IOError as e:
        if e.errno == errno.ENOENT:
            cache_logger.debug('%s not in cache', cache_key)
            raise KeyError('not in cache')

def downloadToCache(stream, hashinfo={}, cache_key=None):
    ''' Download the specified stream to a temporary cache directory, and
        return (path to the downloaded, cache key).
        If cache_key is None, then a cache key will be generated and returned,
        but you will probably want to remove the cache file yourself (this is
        safe).
    '''
    hash_name  = None
    hash_value = None
    m = None

    if len(hashinfo):
        # check for hashes in preferred order. Currently this is just sha256
        # (which the registry uses). Initial investigations suggest that github
        # doesn't return a header with the hash of the file being downloaded.
        for h in ('sha256',):
            if h in hashinfo:
                hash_name  = h
                hash_value = hashinfo[h]
                m = getattr(hashlib, h)()
                break
        if not hash_name:
            logger.warning('could not find supported hash type in %s', hashinfo)

    if cache_key is None:
        cache_key = '%032x' % random.getrandbits(256)

    cache_dir = folders.cacheDirectory()
    fsutils.mkDirP(cache_dir)
    cache_as = os.path.join(cache_dir, cache_key)

    (download_file, download_fname) = tempfile.mkstemp(prefix=cache_dir)
    with os.fdopen(download_file, 'wb') as f:
        f.seek(0)
        for chunk in stream.iter_content(4096):
            f.write(chunk)
            if hash_name:
                m.update(chunk)

        if hash_name:
            calculated_hash = m.hexdigest()
            logger.debug(
                'calculated %s hash: %s check against: %s' % (
                    hash_name, calculated_hash, hash_value
                )
            )
            if hash_value and (hash_value != calculated_hash):
                raise Exception('Hash verification failed.')
        logger.debug('wrote tarfile of size: %s to %s', f.tell(), download_fname)
        f.truncate()
    try:
        os.rename(download_fname, cache_as)
    except OSError as e:
        if e.errno == errno.ENOENT:
            # if we failed, it's because the file already exists (probably
            # because another process got there first), so just rm our
            # temporary file and continue
            cache_logger.debug('another process downloaded %s first', cache_key)
            fsutils.rmF(download_fname)
        else:
            raise

    return (cache_as, cache_key)

@sometimesPruneCache(0.05)
def unpackTarballStream(stream, into_directory, hash={}, cache_key=None):
    ''' Unpack a responses stream that contains a tarball into a directory. If
        a hash is provided, then it will be used as a cache key (for future
        requests you can try to retrieve the key value from the cache first,
        before making the request)
    '''

    download_fname, cache_key = downloadToCache(stream, hash, cache_key)
    unpackFromCache(cache_key, into_directory)

    # if we didn't provide a cache key, there's no point in storing the cache
    if cache_key is None:
        fsutils.rmF(download_fname)





# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import os
import errno
import shutil
import stat

def mkDirP(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise

def rmF(path):
    try:
        if isLink(path):
            rmLink(path)
        else:
            os.remove(path)
    except OSError as exception:
        if exception.errno != errno.ENOENT:
            raise

def _rmRfNoRetry(path):
    # we may have to make files writable before we can successfully delete
    # them, to do this
    def fixPermissions(fn, path, excinfo):
        if os.access(path, os.W_OK):
            raise
        else:
            os.chmod(path, stat.S_IWUSR)
            fn(path)
    try:
        if isLink(path):
            rmLink(path)
        elif os.path.isfile(path):
            rmF(path)
        else:
            shutil.rmtree(path, onerror=fixPermissions)
    except OSError as exception:
        if exception.errno == errno.ENOTDIR:
            rmF(path)
        elif exception.errno != errno.ENOENT:
            raise

def rmRf(path):
    # on windows, it seems that various system processes (antivirus, search
    # indexing, and possibly other things) seem to keep files "open" after
    # python has closed them, preventing them from being removed if we
    # later (in the same process) try to remove them ...
    for x in range(0, 100):
        try:
            _rmRfNoRetry(path)
            break
        # ... ultimately leading to this error ...
        except OSError as e:
            if getattr(__builtins__, "WindowsError", None) is not None:
                # 145 = Directory not empty
                if isinstance(e, WindowsError):
                    if e.errno == 145: #pylint: disable=undefined-variable
                        continue
                        # ... trying again should fix the problem
            # in all other cases, raise the exception
            raise


def fullySplitPath(path):
    components = []
    while True:
        part, component = os.path.split(path)
        if part == path:
            # absolute path
            components.append(part)
            break
        elif component == path:
            components.append(component)
            break
        else:
            components.append(component)
            path = part
    components.reverse()
    return components

# Some functions are platform-dependent
_platform_fsutils = __import__("fsutils_win" if os.name == 'nt' else "fsutils_posix", globals(), locals(), ['*'])
isLink        = _platform_fsutils.isLink
tryReadLink   = _platform_fsutils.tryReadLink
_symlink      = _platform_fsutils._symlink
realpath      = _platform_fsutils.realpath
dropRootPrivs = _platform_fsutils.dropRootPrivs
rmLink        = _platform_fsutils.rmLink
which         = _platform_fsutils.which

# !!! FIXME: the logic in the "except" block below probably doesn't work in Windows
def symlink(source, link_name):
    try:
        # os.symlink doesn't update existing links, so need to rm first
        rmF(link_name)
        _symlink(source, link_name)
    except OSError as exception:
        if exception.errno != errno.EEXIST and (tryReadLink(link_name) != source):
            raise



# Copyright 2014-2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import argparse

# Github Access, , access repositories on github, internal
from .lib import auth, registry_access

def addOptions(parser):
    # set the registry URL to save an API key for (this argument is also a
    # top-level one, use a different dest here so we can distinguish)
    parser.add_argument('--registry', dest='_registry', help=argparse.SUPPRESS)

def execCommand(args, following_args):
    registry = args._registry or args.registry
    auth.deauthorize()
    registry_access.deauthorize(registry)




# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

from multiprocessing.pool import ThreadPool

# this module just provides a shared thread pool for all multiprocessing users
# to use: the number of threads is high because tasks are expected to be
# IO-bound

pool = ThreadPool(4)




#!/usr/bin/env python
# Copyright 2015 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.

# standard library modules, , ,
import unittest
import os
import tempfile

# internal modules:
from yotta.lib.fsutils import mkDirP, rmRf
from . import cli

Test_Outdated = {
'module.json':'''{
  "name": "test-outdated",
  "version": "0.0.0",
  "description": "Test yotta outdated",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0",
  "dependencies":{
    "test-testing-dummy": "*"
  }
}''',
'source/foo.c':'''#include "stdio.h"
int foo(){
    printf("foo!\\n");
    return 7;
}''',
# test-testing-dummy v0.0.1 (a newer version is available from the registry,
# and will be installed by yt up)
'yotta_modules/test-testing-dummy/module.json':'''{
  "name": "test-testing-dummy",
  "version": "0.0.1",
  "description": "Test yotta's compilation of tests.",
  "author": "James Crosby <james.crosby@arm.com>",
  "license": "Apache-2.0"
}
'''
}

class TestCLIUpdate(unittest.TestCase):
    def writeTestFiles(self, files, add_space_in_path=False):
        test_dir = tempfile.mkdtemp()
        if add_space_in_path:
            test_dir = test_dir + ' spaces in path'

        for path, contents in files.items():
            path_dir, file_name =  os.path.split(path)
            path_dir = os.path.join(test_dir, path_dir)
            mkDirP(path_dir)
            with open(os.path.join(path_dir, file_name), 'w') as f:
                f.write(contents)
        return test_dir

    def test_update(self):
        path = self.writeTestFiles(Test_Outdated, True)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', 'update'], cwd=path)
        self.assertEqual(statuscode, 0)
        self.assertIn('download test-testing-dummy', stdout + stderr)

        rmRf(path)

    def test_updateExplicit(self):
        path = self.writeTestFiles(Test_Outdated, True)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', 'update', 'test-testing-dummy'], cwd=path)
        self.assertEqual(statuscode, 0)
        self.assertIn('download test-testing-dummy', stdout + stderr)

        rmRf(path)

    def test_updateNothing(self):
        path = self.writeTestFiles(Test_Outdated, True)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', 'up'], cwd=path)
        self.assertEqual(statuscode, 0)
        self.assertIn('download test-testing-dummy', stdout + stderr)

        stdout, stderr, statuscode = cli.run(['-t', 'x86-linux-native', 'up'], cwd=path)
        self.assertEqual(statuscode, 0)
        self.assertNotIn('download test-testing-dummy', stdout + stderr)

        rmRf(path)



#!/usr/bin/env python
# Copyright 2014 ARM Limited
#
# Licensed under the Apache License, Version 2.0
# See LICENSE file for details.


# standard library modules, , ,
import unittest

# sourceparse, , parse version source urls, internal
from yotta.lib import sourceparse


Registry_URLs = [
    '',
    '*',
    '1.2.3',
    '>=1.2.3',
    '^0.1.2',
    '~0.1.2',
]

Github_URLs = [
    'username/reponame',
    'username/reponame#1.2.3',
    'username/reponame#^1.2.3',
    'username/reponame#-1.2.3',
    'username/reponame#branch-or-tag-name',
    'username/reponame@1.2.3',
    'username/reponame@^1.2.3',
    'username/reponame@-1.2.3',
    'username/reponame@branch-or-tag-name',

]

Git_URLs = [
    'git+ssh://somewhere.com/something/etc/etc',
    'git+ssh://somewhere.com/something/etc/etc#1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#^1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#~1.2.3',
    'git+ssh://somewhere.com/something/etc/etc#branch-name',
    'ssh://somewhere.com/something/etc/etc.git',
    'ssh://somewhere.com/something/etc/etc.git#^1.2.3',
    'ssh://somewhere.com/something/etc/etc.git#~1.2.3',
    'ssh://somewhere.com/something/etc/etc.git#branch-name',
    'http://somewhere.something/something.git',
]

HG_URLs = [
    'hg+ssh://somewhere.com/something/etc/etc',
    'hg+ssh://somewhere.com/something/etc/etc#1.2.3',
    'hg+ssh://somewhere.com/something/etc/etc#^1.2.3',
    'hg+ssh://somewhere.com/something/etc/etc#~1.2.3',
    'ssh://somewhere.com/something/etc/etc.hg',
    'ssh://somewhere.com/something/etc/etc.hg#^1.2.3',
    'ssh://somewhere.com/something/etc/etc.hg#~1.2.3',
    'http://somewhere.something/something.hg',
]

test_invalid_urls = [
    'stuff otherstuff',
    'somthing/somethingelse/somethingelseagain'
]


class TestGitAccess(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_registryURLs(self):
        for url in Registry_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'registry')

    def test_githubURLs(self):
        for url in Github_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'github')

    def test_gitURLs(self):
        for url in Git_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'git')

    def test_hgURLs(self):
        for url in HG_URLs:
            sv = sourceparse.parseSourceURL(url)
            self.assertEqual(sv.source_type, 'hg')

if __name__ == '__main__':
    unittest.main()


